package com.ruoyi.web.controller.staff;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.store.domain.StoreBO;
import com.ruoyi.ss.store.domain.StoreQuery;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.storeStaff.domain.enums.StoreStaffPermissions;
import com.ruoyi.ss.storeStaff.service.StoreStaffValidator;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 员工 店铺Controller
 * @author wjh
 * 2024/11/23
 */
@RestController
@RequestMapping("/staff/store")
public class StaffStoreController extends BaseController {

    @Autowired
    private StoreService storeService;

    @Autowired
    private StoreStaffValidator storeStaffValidator;

    @Autowired
    private StoreValidator storeValidator;

    @ApiOperation("员工获取管理的店铺列表")
    @GetMapping("/list")
    public TableDataInfo list(StoreQuery query) {
        startPage();
        query.setStaffUserId(getUserId());
        query.setStaffPermission(StoreStaffPermissions.STORE_VIEW.getCode());
        return getDataTable(storeService.selectStaffStoreList(query));
    }

    @ApiOperation("员工获取店铺详情")
    @GetMapping("/{storeId}")
    public AjaxResult getDetail(@PathVariable Long storeId) {
        StoreQuery query = new StoreQuery();
        query.setStaffUserId(getUserId());
        query.setStaffPermission(StoreStaffPermissions.STORE_VIEW.getCode());
        query.setStoreId(storeId);
        return success(storeService.selectStaffStoreOne(query));
    }

    // TODO 建议还是合在商户的接口中使用
    @ApiOperation("员工申请修改店铺信息")
    @PutMapping
    public AjaxResult update(@RequestBody @Validated(ValidGroup.FrontUpdate.class) StoreBO data) {
        if (!storeStaffValidator.canOperaStore(data.getStoreId(), getUserId())) {
            return error("您无权修改该店铺");
        }
        data = data.filterUpdateByAppStaff();
        ServiceUtil.assertion(storeValidator.preUpdateByApp(data));
        return toAjax(storeService.updateApply(data));
    }

}
